/* $LAAS: improc.c 2008/07/01 19:08:49 mallet $ */

/*
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Tue Jul  1 2008
 */

static const char _rcsid[] = "$LAAS: improc.c 2008/07/01 19:08:49 mallet $";

#include <stdio.h>
#include <math.h>
#include <err.h>

#include "viam_private.h"

#ifdef VIAM_OPENCV_SUPPORT
# ifndef CV_INLINE
#  define CV_INLINE	static inline
# endif
# define CV_NO_BACKWARD_COMPATIBILITY
# include <cxtypes.h>
# include <cv.h>
#endif


/* --- viam_improc_convolve ------------------------------------------ */

#ifdef VIAM_OPENCV_SUPPORT

/** \brief Apply Laplacian of Gaussian filter on image.
 *
 * \param[in] vh	Library handle
 * \param[in] image	Image used for initial acquisition
 * \param[in] source	Source image
 * \param[out] result	Resulting image
 * \param[in] args	Filter arguments
 * \param[in] data	Filter private data
 *
 * \return 0 on success.
 */

int
viam_improc_log(viam_handle_t vh, viam_image_t camera,
		const IplImage *source, IplImage *result,
		viam_filterargs_t *args, void *data)
{
  CvMat _kernel;

  switch((int)args->value) {
    case 1 ... 4: {
      static const int logkernel[] = {
	0,  -1,  0,
	-1, 4,  -1,
	0,  -1,  0,
      };
      _kernel = cvMat(3, 3, CV_32SC1, (int *)logkernel);
      break;
    }

    case 5 ... 8: {
      static const int logkernel[] = {
	0,  0,   -1,  0,  0,
	0,  -1,   -2,  -1,  0,
	-1,  -2, 16,  -2,  -1,
	0,  -1,   -2,  -1,  0,
	0,  0,   -1,  0,  0
      };
      _kernel = cvMat(5, 5, CV_32SC1, (int *)logkernel);
      break;
    }

    case 9:
    default: {
      static const float logkernel[] = {
	0, 0, -.15, -.1, -.1, -.1, -.15, 0, 0,
	0, -.1, -.15, -.25, -.25, -.25, -.15, -.1, 0,
	-.15, -.15, -.25, -.15, 0, -.15, -.25, -.15, -.15,
	-.1, -.25, -.15, 0.6, 1.2, 0.6, -.15, -.25, -.1,
	-.1, -.25, 0, 1.2, 2.0, 1.2, 0, -.25, -.1,
	-.1, -.25, -.15, 0.6, 1.2, 0.6, -.15, -.25, -.1,
	-.15, -.15, -.25, -.15, 0, -.15, -.25, -.15, -.15,
	0, -.1, -.15, -.25, -.25, -.25, -.15, -.1, 0,
	0, 0, -.15, -.1, -.1, -.1, -.15, 0, 0,
      };
      _kernel = cvMat(9, 9, CV_32FC1, (double *)logkernel);
      break;
    }
  }

  int s = 0;

  cvFilter2D(source, result, &_kernel, cvPoint(-1,-1));
  if (cvGetErrStatus()) s = Eviamlib_OPENCV;

  return s;
}

#endif /* VIAM_OPENCV_SUPPORT */
